LocationDataModel class
Constructors
Properties
- accuracy → double?
-
final
- altitude → double?
-
final
- hashCode → int
-
The hash code for this object.
no setterinherited
- heading → double?
-
final
- id → int?
-
final
- latitude → double
-
final
- longitude → double
-
final
- runtimeType → Type
-
A representation of the runtime type of the object.
no setterinherited
- speed → double?
-
final
- timestamp → DateTime
-
final
Methods
-
noSuchMethod(
Invocation invocation) → dynamic -
Invoked when a nonexistent method or property is accessed.
inherited
-
toJson(
) → Map< String, dynamic> -
toMap(
) → Map< String, dynamic> -
toString(
) → String -
A string representation of this object.
override
Operators
-
operator ==(
Object other) → bool -
The equality operator.
inherited